{% load render_table from django_tables2 %}
{% include 'utilities/obj_table.html' with table=related_session_table table_template='panel_table.html' heading='Related BGP Sessions' panel_class='default noprint' %}